@@ -25,6 +25,7 @@ from ._reporting import (
     EXPERIMENT_CREATION_FAILED,
 )
 from ._typing import Any, Dict, Optional
+from ._ui import UI
 from .api import API, APIExperiment
 from .artifacts import (
     Artifact,
@@ -33,8 +34,9 @@ from .artifacts import (
     _log_artifact,
     _parse_artifact_name,
 )
-from .comet import Streamer, format_url, generate_guid, is_valid_experiment_key
+from .comet import Streamer, format_url, is_valid_experiment_key
 from .config import (
+    discard_api_key,
     get_api_key,
     get_config,
     get_global_experiment,
@@ -62,6 +64,7 @@ from .exceptions import (
 )
 from .experiment import BaseExperiment
 from .feature_toggles import HTTP_LOGGING, FeatureToggles
+from .heartbeat import HeartbeatThread
 from .json_encoder import NestedEncoder
 from .loggers.fastai_logger import patch as fastai_patch
 from .loggers.keras_logger import patch as keras_patch
@@ -95,6 +98,7 @@ from .rpc import create_remote_call, get_remote_action_definition
 from .utils import (
     Embedding,
     Histogram,
+    generate_guid,
     get_comet_version,
     get_time_monotonic,
     make_template_filename,
@@ -102,6 +106,8 @@ from .utils import (
     valid_ui_tabs,
 )
 
+ui = UI()
+
 __author__ = "Gideon<Gideon@comet.ml>"
 __all__ = [
     "API",
@@ -288,6 +294,7 @@ class Experiment(BaseExperiment):
         self.ws_connection = None  # type: Optional[WebSocketConnection]
         self.connection = None  # type: Optional[RestServerConnection]
         self.rest_api_client = None  # type: Optional[RestApiClient]
+        self._heartbeat_thread = None  # type: Optional[HeartbeatThread]
 
         if self.disabled is not True:
             self._start()
@@ -357,6 +364,19 @@ class Experiment(BaseExperiment):
         # Initiate the streamer
         self._initialize_streamer(full_ws_url, initial_offset)
 
+        # Initiate the heartbeat thread
+        self._heartbeat_thread = HeartbeatThread(
+            INITIAL_BEAT_DURATION / 1000.0,
+            self.connection,
+            self._on_pending_rpcs_callback,
+        )
+
+        # setup parameters update callback
+        self._heartbeat_thread.on_parameters_update_interval_callback = (
+            self.streamer.parameters_batch.update_interval
+        )
+
+        self._heartbeat_thread.start()
         return True
 
     def _authenticate(self):
@@ -365,23 +385,30 @@ class Experiment(BaseExperiment):
         various parameters and settings
         """
         # Get an id for this run
-        run_id_results = self.connection.get_run_id(self.project_name, self.workspace)
+        try:
+            run_id_results = self.connection.get_run_id(
+                self.project_name, self.workspace
+            )
 
-        (
-            self.run_id,
-            ws_server,
-            self.project_id,
-            self.is_github,
-            self.focus_link,
-            self.upload_limit,
-            self.asset_upload_limit,
-            feature_toggles,
-            initial_offset,
-            self.upload_web_asset_url_prefix,
-            self.upload_web_image_url_prefix,
-            self.upload_api_asset_url_prefix,
-            self.upload_api_image_url_prefix,
-        ) = run_id_results
+            (
+                self.run_id,
+                ws_server,
+                self.project_id,
+                self.is_github,
+                self.focus_link,
+                self.upload_limit,
+                self.asset_upload_limit,
+                feature_toggles,
+                initial_offset,
+                self.upload_web_asset_url_prefix,
+                self.upload_web_image_url_prefix,
+                self.upload_api_asset_url_prefix,
+                self.upload_api_image_url_prefix,
+            ) = run_id_results
+        except InvalidAPIKey as e:
+            # discard invalid API key
+            discard_api_key(e.api_key)
+            raise e
 
         self.feature_toggles = FeatureToggles(feature_toggles, self.config)
 
@@ -424,6 +451,30 @@ class Experiment(BaseExperiment):
         except Exception:
             LOGGER.error("Failed to report experiment status", exc_info=True)
 
+    def _start_gpu_thread(self):
+        super(Experiment, self)._start_gpu_thread()
+
+        if not self.alive:
+            return
+
+        # Connect heartbeat thread and the gpu thread
+        if self._heartbeat_thread is not None and self.gpu_thread is not None:
+            self._heartbeat_thread.on_gpu_monitor_interval_callback = (
+                self.gpu_thread.update_interval
+            )
+
+    def _start_cpu_thread(self):
+        super(Experiment, self)._start_cpu_thread()
+
+        if not self.alive:
+            return
+
+        # Connect heartbeat thread and the cpu thread
+        if self._heartbeat_thread is not None and self.cpu_thread is not None:
+            self._heartbeat_thread.on_cpu_monitor_interval_callback = (
+                self.cpu_thread.update_interval
+            )
+
     def _mark_as_ended(self):
         if self.alive:
             try:
@@ -437,7 +488,7 @@ class Experiment(BaseExperiment):
         self.connection.report(*args, **kwargs)
 
     def _on_end(self, wait=True):
-        """ Called when the Experiment is replaced by another one or at the
+        """Called when the Experiment is replaced by another one or at the
         end of the script
         """
         successful_clean = super(Experiment, self)._on_end(wait=wait)
@@ -461,6 +512,11 @@ class Experiment(BaseExperiment):
                         successful_clean = False
                         self.ws_connection.force_close()
 
+            if self._heartbeat_thread is not None:
+                self._heartbeat_thread.close()
+                if wait is True:
+                    self._heartbeat_thread.join(10)
+
             if self.connection is not None:
                 self.connection.close()
 
@@ -511,8 +567,7 @@ class Experiment(BaseExperiment):
         return ""
 
     def _on_pending_rpcs_callback(self):
-        """ Called by streamer when we have pending rpcs
-        """
+        """Called by streamer when we have pending rpcs"""
         LOGGER.debug("Checking pending rpcs")
         calls = self.connection.get_pending_rpcs()["remoteProcedureCalls"]
 
@@ -783,10 +838,13 @@ class Experiment(BaseExperiment):
         return _log_artifact(artifact, self)
 
     def get_artifact(
-        self, artifact_name, workspace=None, version_or_alias=None,
+        self,
+        artifact_name,
+        workspace=None,
+        version_or_alias=None,
     ):
         # type: (str, Optional[str], Optional[str]) -> LoggedArtifact
-        """ Returns a logged artifact object that can be used to access the artifact version assets and
+        """Returns a logged artifact object that can be used to access the artifact version assets and
         download them locally.
 
         If no version or alias is provided, the latest version for that artifact is returned.
@@ -1074,6 +1132,8 @@ class ExistingExperiment(Experiment):
 
         except InvalidAPIKey as e:
             LOGGER.error(INVALID_API_KEY, e.api_key, exc_info=True)
+            # discarding the invalid API key from the blacklisted strings
+            discard_api_key(e.api_key)
             return
 
         except ValueError:
