@@ -43,7 +43,7 @@ class TracebackLessFormatter(logging.Formatter):
         super(TracebackLessFormatter, self).__init__(*args, **kwargs)
 
         self.hide_traceback = hide_traceback
-        self.blacklist_strings = set()
+        self.strings_to_redact = set()
 
     def format(self, record):
 
@@ -64,7 +64,7 @@ class TracebackLessFormatter(logging.Formatter):
 
         result = super(TracebackLessFormatter, self).format(new_record)
         # If s is not in result, it's faster to check first before doing the substring replacement
-        for s in self.blacklist_strings:
+        for s in self.strings_to_redact:
             # Avoid redacting strings if they are passed explicitely as log records
             if s in result and s not in record.args:
                 result = result.replace(s, self.REDACTED_STRING)
@@ -72,7 +72,7 @@ class TracebackLessFormatter(logging.Formatter):
 
 
 def shorten_record_name(record_name):
-    """ Return the first part of the record (which can be None, comet or
+    """Return the first part of the record (which can be None, comet or
     comet.connection)
     """
     if record_name is None:
@@ -267,11 +267,24 @@ class CometLoggingConfig(object):
         self.root.setLevel(logger_level)
 
     def redact_string(self, string):
+        # type: (str) -> None
+        """Blacklists specified string from logging. The string will be masked in the logging."""
         if self.console_formatter:
-            self.console_formatter.blacklist_strings.add(string)
+            self.console_formatter.strings_to_redact.add(string)
 
         if self.file_formatter:
-            self.file_formatter.blacklist_strings.add(string)
+            self.file_formatter.strings_to_redact.add(string)
+
+    def discard_string_from_redact(self, string):
+        # type: (str) -> None
+        """Discards specified string from the set of blacklisted strings. After this method invocation the string will
+        no longer be masked from the logger output.
+        """
+        if self.console_formatter:
+            self.console_formatter.strings_to_redact.discard(string)
+
+        if self.file_formatter:
+            self.file_formatter.strings_to_redact.discard(string)
 
 
 COMET_LOGGING_CONFIG = CometLoggingConfig()
