@@ -27,6 +27,7 @@ from .._typing import (
     Tuple,
 )
 from ..experiment import BaseExperiment
+from ..logging_messages import TF_KERAS_CALLBACK_WARNING_CLOSED_EXPERIMENT
 from ..predictor import Predictor
 from ..utils import Histogram, get_time_monotonic, tensor_length
 
@@ -373,13 +374,19 @@ def build_base_callback(base):
 
 def build_predictive_early_stopping_keras_callback(base):
     class PredictiveEarlyStoppingKerasCallback(base):  # type: ignore
-        def __init__(self, predictor, predict_epoch_rate=1):
-            # type: (Predictor, int) -> None
+        def __init__(self, experiment, predictor, predict_epoch_rate=1):
+            # type: (BaseExperiment, Predictor, int) -> None
             super(PredictiveEarlyStoppingKerasCallback, self).__init__()
             self.predictor = predictor
             self.predict_epoch_rate = predict_epoch_rate
+            self.experiment = experiment
 
         def on_epoch_end(self, epoch, logs={}):
+            if check_experiment_closed_and_warn(
+                self.experiment, callback_method_name="on_epoch_end"
+            ):
+                return
+
             if epoch % self.predict_epoch_rate == 0:
                 loss = logs.get(self.predictor.loss_name)
                 self.predictor.report_loss(loss)
@@ -401,7 +408,7 @@ def build_empty_keras_callback(base):
 
 def build_keras_callback(base):
     class KerasCallback(base):  # type: ignore
-        """ Keras callback to report params, metrics to Comet.ml Experiment()"""
+        """Keras callback to report params, metrics to Comet.ml Experiment()"""
 
         def __init__(
             self,
@@ -454,6 +461,11 @@ def build_keras_callback(base):
             self.activation_ignore_list = ["flatten", "dropout", "activation"]
 
         def on_epoch_begin(self, epoch, logs=None):
+            if check_experiment_closed_and_warn(
+                self.experiment, callback_method_name="on_epoch_begin"
+            ):
+                return
+
             try:
                 # This function should only be called during train mode.
                 LOGGER.debug("On epoch begin %s %s", epoch, logs)
@@ -469,6 +481,11 @@ def build_keras_callback(base):
                 )
 
         def on_epoch_end(self, epoch, logs=None):
+            if check_experiment_closed_and_warn(
+                self.experiment, callback_method_name="on_epoch_end"
+            ):
+                return
+
             try:
                 # This function should only be called during train mode.
                 LOGGER.debug("On epoch end %s %s", epoch, logs)
@@ -737,6 +754,9 @@ def build_keras_callback(base):
             return None
 
         def on_batch_begin(self, batch, logs=None):
+            if check_experiment_closed_and_warn(self.experiment, show_warning=False):
+                return
+
             try:
                 # This function called directly when in train mode.
                 LOGGER.debug("On batch begin %s %s", batch, logs)
@@ -750,6 +770,9 @@ def build_keras_callback(base):
             """
             Logs training metrics.
             """
+            if check_experiment_closed_and_warn(self.experiment, show_warning=False):
+                return
+
             try:
                 # This function called directly when in train mode.
                 LOGGER.debug("On batch end %s %s", batch, logs)
@@ -797,6 +820,11 @@ def build_keras_callback(base):
             """
             Sets model graph.
             """
+            if check_experiment_closed_and_warn(
+                self.experiment, callback_method_name="on_train_begin"
+            ):
+                return
+
             try:
                 LOGGER.debug("On train begin %s", logs)
 
@@ -845,6 +873,11 @@ def build_keras_callback(base):
                 )
 
         def on_train_end(self, *args, **kwargs):
+            if check_experiment_closed_and_warn(
+                self.experiment, callback_method_name="on_train_end"
+            ):
+                return
+
             try:
                 LOGGER.debug("On train end %r", locals())
             except Exception:
@@ -868,3 +901,21 @@ def get_keras_model(experiment, model):
         return json_model
     else:
         return model
+
+
+def check_experiment_closed_and_warn(
+    experiment, show_warning=True, callback_method_name="undefined"
+):
+    # type: (BaseExperiment, Optional[bool], Optional[str]) -> bool
+    """Checks if specified experiment is closed and display warning
+    Args:
+        experiment: BaseExperiment - the experiment to be checked
+        show_warning: Optional. bool - the flag to indicate if warning should be displayed [default: True]
+        callback_method_name: Optional. string - the name of callback method related to this check.
+    """
+    if not experiment.alive and show_warning:
+        LOGGER.warning(
+            TF_KERAS_CALLBACK_WARNING_CLOSED_EXPERIMENT, callback_method_name
+        )
+
+    return not experiment.alive
