@@ -31,7 +31,8 @@ except ImportError as exc:
         import tensorflow.keras as keras
     except ImportError:
         LOGGER.warning(
-            TF_KERAS_FALLBACK_FAILED, exc_info=True,
+            TF_KERAS_FALLBACK_FAILED,
+            exc_info=True,
         )
         raise exc
 
