@@ -26,7 +26,6 @@ import sys
 import tempfile
 import threading
 import time
-import uuid
 from os.path import basename, splitext
 
 from six.moves.queue import Empty, Queue
@@ -34,21 +33,31 @@ from six.moves.urllib.parse import urlencode, urlsplit, urlunsplit
 
 from ._reporting import ON_EXIT_DIDNT_FINISH_UPLOAD_SDK
 from ._typing import Any, Dict, List, Optional, Tuple
+from .batch_utils import ParametersBatch
 from .compat_utils import json_dump
 from .config import (
     ADDITIONAL_STREAMER_UPLOAD_TIMEOUT,
     DEFAULT_FILE_UPLOAD_READ_TIMEOUT,
+    DEFAULT_PARAMETERS_UPDATE_INTERVAL_SECONDS,
     DEFAULT_STREAMER_MSG_TIMEOUT,
 )
-from .connection import FileUploadManager, format_messages_for_ws
+from .connection import (
+    FileUploadManager,
+    FileUploadManagerMonitor,
+    format_messages_for_ws,
+)
 from .exceptions import CometRestApiException
 from .file_uploader import is_user_text
 from .json_encoder import NestedEncoder
 from .logging_messages import (
     CLOUD_DETAILS_MSG_SENDING_ERROR,
+    FILE_UPLOADS_PROMPT,
     MODEL_GRAPH_MSG_SENDING_ERROR,
     OS_PACKAGE_MSG_SENDING_ERROR,
+    STREAMER_CLOSED_PUT_MESSAGE_FAILED,
+    STREAMER_FAILED_TO_PROCESS_ALL_MESSAGES,
     STREAMER_WAIT_FOR_FINISH_FAILED,
+    WAITING_DATA_UPLOADED,
 )
 from .messages import (
     BaseMessage,
@@ -65,7 +74,7 @@ from .messages import (
 from .utils import (
     data_to_fp,
     log_once_at_level,
-    wait_for_empty,
+    wait_for_done,
     write_file_like_to_tmp_file,
 )
 
@@ -83,6 +92,8 @@ class BaseStreamer(threading.Thread):
         self.counter = initial_offset
         self.messages = Queue()  # type: Queue
         self.queue_timeout = queue_timeout
+        self.closed = False
+        self.__lock__ = threading.RLock()
 
         LOGGER.debug("%r instantiated with duration %s", self, self.queue_timeout)
 
@@ -91,9 +102,28 @@ class BaseStreamer(threading.Thread):
         Puts a message in the queue
         :param message: Some kind of payload, type agnostic
         """
-        if message is not None:
-            LOGGER.debug("Putting 1 %r in queue", message.__class__)
-            self.messages.put(message)
+        with self.__lock__:
+            if message is not None:
+                if not self.closed:
+                    LOGGER.debug("Putting 1 %r in queue", message.__class__)
+                    self.messages.put(message)
+                else:
+                    LOGGER.debug(STREAMER_CLOSED_PUT_MESSAGE_FAILED)
+                    LOGGER.debug("Ignored message (streamer closed): %s", message)
+
+    def close(self):
+        """
+        Puts a None in the queue which leads to closing it.
+        """
+        with self.__lock__:
+            if self.closed is True:
+                LOGGER.debug("Streamer tried to be closed more than once")
+                return
+
+            # Send a message to close
+            self.put_message_in_q(CloseMessage())
+
+            self.closed = True
 
     def _before_run(self):
         pass
@@ -181,6 +211,8 @@ class Streamer(BaseStreamer):
         msg_waiting_timeout=DEFAULT_STREAMER_MSG_TIMEOUT,
         file_upload_waiting_timeout=ADDITIONAL_STREAMER_UPLOAD_TIMEOUT,
         file_upload_read_timeout=DEFAULT_FILE_UPLOAD_READ_TIMEOUT,
+        wait_for_finish_sleep_interval=15,
+        parameters_batch_base_interval=DEFAULT_PARAMETERS_UPDATE_INTERVAL_SECONDS,
     ):
         super(Streamer, self).__init__(initial_offset, beat_duration / 1000.0)
         self.daemon = True
@@ -189,7 +221,6 @@ class Streamer(BaseStreamer):
         self.connection = connection
         self.rest_api_client = rest_api_client
 
-        self.closed = False
         self.stop_processing = False
         self.on_gpu_monitor_interval = None
         self.on_cpu_monitor_interval = None
@@ -198,6 +229,7 @@ class Streamer(BaseStreamer):
 
         self.last_beat = time.time()
         self.msg_waiting_timeout = msg_waiting_timeout
+        self.wait_for_finish_sleep_interval = wait_for_finish_sleep_interval
         self.file_upload_waiting_timeout = file_upload_waiting_timeout
         self.file_upload_read_timeout = file_upload_read_timeout
 
@@ -208,21 +240,10 @@ class Streamer(BaseStreamer):
         self.run_id = run_id
         self.project_id = project_id
 
+        self.parameters_batch = ParametersBatch(parameters_batch_base_interval)
+
         LOGGER.debug("Streamer instantiated with ws url %s", self.ws_connection)
 
-    def close(self):
-        """
-        Puts a None in the queue which leads to closing it.
-        """
-        if self.closed is True:
-            LOGGER.debug("Streamer tried to be closed more than once")
-            return
-
-        # Send a message to close
-        self.put_message_in_q(CloseMessage())
-
-        self.closed = True
-
     def _before_run(self):
         self.ws_connection.wait_for_connection()
 
@@ -265,12 +286,12 @@ class Streamer(BaseStreamer):
                         elif isinstance(message, CloudDetailsMessage):
                             self._process_cloud_details_message(message)
                         else:
-                            raise ValueError("Unkown message type %r", message)
+                            raise ValueError("Unknown message type %r", message)
+
+                # attempt to send collected parameters
+                if self.parameters_batch.accept(self._send_ws_message):
+                    LOGGER.debug("Parameters batch was sent")
 
-                try:
-                    self._check_heartbeat()
-                except Exception:
-                    LOGGER.debug("Heartbeat error", exc_info=True)
             else:
                 LOGGER.debug("WS connection not ready")
                 # Basic backoff
@@ -296,6 +317,7 @@ class Streamer(BaseStreamer):
             upload_endpoint=url,
             on_asset_upload=message._on_asset_upload,
             on_failed_asset_upload=message._on_failed_asset_upload,
+            estimated_size=message._size,
         )
         LOGGER.debug("Processing uploading message done")
 
@@ -316,6 +338,7 @@ class Streamer(BaseStreamer):
             upload_endpoint=url,
             on_asset_upload=message._on_asset_upload,
             on_failed_asset_upload=message._on_failed_asset_upload,
+            estimated_size=message._size,
         )
         LOGGER.debug("Processing in-memory uploading message done")
 
@@ -336,26 +359,40 @@ class Streamer(BaseStreamer):
             upload_endpoint=url,
             on_asset_upload=message._on_asset_upload,
             on_failed_asset_upload=message._on_failed_asset_upload,
+            estimated_size=message._size,
         )
         LOGGER.debug("Processing remote uploading message done")
 
-    def _process_ws_message(self, message, offset):
+    def _send_ws_message(self, message, offset):
         # type: (Message, int) -> None
+        """The method to be used as callback for parameters batching processing"""
         try:
-            message_dict = message._non_null_dict()
-
-            # Inject online specific values
-            message_dict["apiKey"] = self.api_key
-            message_dict["runId"] = self.run_id
-            message_dict["projectId"] = self.project_id
-            message_dict["experimentKey"] = self.experiment_key
-            message_dict["offset"] = offset
-
-            data = format_messages_for_ws([message_dict])
+            data = self._serialise_message_for_ws(message=message, offset=offset)
             self.ws_connection.send(data)
         except Exception:
             LOGGER.debug("WS sending error", exc_info=True)
 
+    def _process_ws_message(self, message, offset):
+        # type: (Message, int) -> None
+        batched = self.parameters_batch.append(message=message, offset=offset)
+        if not batched:
+            # send message immediately
+            self._send_ws_message(message, offset)
+
+    def _serialise_message_for_ws(self, message, offset):
+        # type: (Message, int) -> str
+        """Enhance provided message with relevant meta-data and serialize it to JSON compatible with WS format"""
+        message_dict = message._non_null_dict()
+
+        # Inject online specific values
+        message_dict["apiKey"] = self.api_key
+        message_dict["runId"] = self.run_id
+        message_dict["projectId"] = self.project_id
+        message_dict["experimentKey"] = self.experiment_key
+        message_dict["offset"] = offset
+
+        return format_messages_for_ws([message_dict])
+
     def _process_os_package_message(self, message):
         # type: (OsPackagesMessage) -> None
         try:
@@ -432,104 +469,49 @@ class Streamer(BaseStreamer):
         except Exception:
             LOGGER.debug("Error sending cloud details message", exc_info=True)
 
-    def _check_heartbeat(self):
-        """
-        Check if we should send an heartbeat
-        """
-        next_beat = self.last_beat + self.queue_timeout
-        now = time.time()
-        if next_beat < now:
-            if self.closed is True:
-                LOGGER.debug("Websocket connection heartbeat while closed")
-
-            LOGGER.debug("Doing an hearbeat")
-            # We need to update the last beat time before doing the actual
-            # call as the call might fails and the last beat would not been
-            # updated. That would trigger a heartbeat for each message.
-            self.last_beat = time.time()
-            new_beat_duration, data, pending_rpcs = self.connection.heartbeat()
-
-            # Handle handshake updates:
-            gpu_monitor_interval = data.get("gpu_monitor_interval")
-            LOGGER.debug(
-                "Getting a new gpu monitor duration %d %r",
-                gpu_monitor_interval,
-                self.on_gpu_monitor_interval,
-            )
-            cpu_monitor_interval = data.get("cpu_monitor_interval")
-            LOGGER.debug(
-                "Getting a new cpu monitor duration %d %r",
-                cpu_monitor_interval,
-                self.on_cpu_monitor_interval,
-            )
-            LOGGER.debug("Getting a new heartbeat duration %d", new_beat_duration)
-            self.queue_timeout = new_beat_duration / 1000.0  # We get milliseconds
-
-            # If we get a callback for a monitor duration, call it:
-            if self.on_gpu_monitor_interval is not None:
-                try:
-                    self.on_gpu_monitor_interval(gpu_monitor_interval / 1000.0)
-                except Exception:
-                    LOGGER.debug(
-                        "Error calling the gpu monitor interval callback", exc_info=True
-                    )
-            if self.on_cpu_monitor_interval is not None:
-                try:
-                    self.on_cpu_monitor_interval(cpu_monitor_interval / 1000.0)
-                except Exception:
-                    LOGGER.debug(
-                        "Error calling the cpu monitor interval callback", exc_info=True
-                    )
-
-            # If there are some pending rpcs
-            if pending_rpcs and self.on_pending_rpcs_callback is not None:
-                try:
-                    self.on_pending_rpcs_callback()
-                except Exception:
-                    LOGGER.debug("Error calling the rpc callback", exc_info=True)
-
     def wait_for_finish(self):
-        """ Blocks the experiment from exiting until all data was sent to server OR the configured timeouts has expired."""
+        """Blocks the experiment from exiting until all data was sent to server
+        OR the configured timeouts has expired."""
 
-        msg = "Uploading metrics, params, and assets to Comet before program termination (may take several seconds)"
         if not self._is_msg_queue_empty():
-            log_once_at_level(logging.INFO, msg)
+            log_once_at_level(logging.INFO, WAITING_DATA_UPLOADED)
             log_once_at_level(
                 logging.INFO,
                 "The Python SDK has %d seconds to finish before aborting...",
                 self.msg_waiting_timeout,
             )
 
-            wait_for_empty(
+            wait_for_done(
                 self._is_msg_queue_empty,
                 self.msg_waiting_timeout,
                 progress_callback=self._show_remaining_messages,
-                sleep_time=5,
+                sleep_time=self.wait_for_finish_sleep_interval,
             )
 
         if not self._is_msg_queue_empty():
-            msg = "Failed to send all messages, metrics and output will likely be incomplete"
-            LOGGER.warning(msg)
+            LOGGER.warning(STREAMER_FAILED_TO_PROCESS_ALL_MESSAGES)
 
         # From now on, stop processing the message queue as it might contains file upload messages
         # TODO: Find a correct way of testing it
         self.stop_processing = True
         self.file_upload_manager.close()
 
+        # Send all remained parameters from the batch if any
+        self.parameters_batch.accept(self._send_ws_message, unconditional=True)
+
         if not self.file_upload_manager.all_done():
-            msg = (
-                "Waiting for completion of the file uploads (may take several seconds)"
-            )
-            LOGGER.info(msg)
+            monitor = FileUploadManagerMonitor(self.file_upload_manager)
+
+            LOGGER.info(FILE_UPLOADS_PROMPT)
             LOGGER.info(
                 "The Python SDK has %d seconds to finish before aborting...",
                 self.file_upload_waiting_timeout,
             )
-            wait_for_empty(
-                self.file_upload_manager.all_done,
+            wait_for_done(
+                monitor.all_done,
                 self.file_upload_waiting_timeout,
-                progress_callback=self.file_upload_manager.log_remaining_uploads,
-                sleep_time=5,
+                progress_callback=monitor.log_remaining_uploads,
+                sleep_time=self.wait_for_finish_sleep_interval,
             )
 
         if not self._is_msg_queue_empty() or not self.file_upload_manager.all_done():
@@ -554,7 +536,11 @@ class Streamer(BaseStreamer):
         finished = self.messages.empty()
 
         if finished is False:
-            LOGGER.debug("Message queue not empty, %d messages", self.messages.qsize())
+            LOGGER.debug(
+                "Message queue not empty, %d messages, closed %s",
+                self.messages.qsize(),
+                self.closed,
+            )
             LOGGER.debug(
                 "WS Connection connected? %s %s",
                 self.ws_connection.is_connected(),
@@ -585,29 +571,10 @@ class OfflineStreamer(BaseStreamer):
     def __init__(self, tmp_dir, initial_offset):
         super(OfflineStreamer, self).__init__(initial_offset, 1)
         self.daemon = True
-        self.closed = False
         self.tmp_dir = tmp_dir
 
         self.file = open(os.path.join(self.tmp_dir, "messages.json"), "wb")
 
-    def close(self):
-        """
-        Puts a None in the queue which leads to closing it.
-        """
-        if self.closed is True:
-            LOGGER.debug("Streamer tried to be closed more than once")
-            return
-
-        # Send a message to close
-        self.put_message_in_q(CloseMessage())
-
-        # We cannot close the file at this moment because their might still be
-        # messages in the queue to write
-
-        self.closed = True
-
-        LOGGER.debug("OfflineStreamer has been closed")
-
     def _write(self, json_line_message):
         # type: (Dict[str, Any]) -> None
         compact_json_dump(json_line_message, self.file)
@@ -707,6 +674,7 @@ class OfflineStreamer(BaseStreamer):
             message.additional_params,
             message.metadata,
             clean=True,
+            size=message._size,
         )
 
         return self._process_upload_message(new_message)
@@ -754,7 +722,7 @@ class OfflineStreamer(BaseStreamer):
         self._write(data)
 
     def wait_for_finish(self):
-        """ Blocks the experiment from exiting until all data was sent to server OR 30 seconds passed."""
+        """Blocks the experiment from exiting until all data was sent to server OR 30 seconds passed."""
 
         msg = "Saving offline stats to disk before program termination (may take several seconds)"
         log_once_at_level(logging.INFO, msg)
@@ -790,8 +758,7 @@ class OfflineStreamer(BaseStreamer):
             return True
 
     def _wait_for_empty(self, timeout, verbose=False, sleep_time=1):
-        """ Wait up to TIMEOUT seconds for the messages queue to be empty
-        """
+        """Wait up to TIMEOUT seconds for the messages queue to be empty"""
         end_time = time.time() + timeout
 
         while not self.messages.empty() and time.time() < end_time:
@@ -802,7 +769,7 @@ class OfflineStreamer(BaseStreamer):
             # before sleep_time has elapsed:
             end_sleep_time = time.time() + sleep_time
             while not self.messages.empty() and time.time() < end_sleep_time:
-                time.sleep(sleep_time / 20)
+                time.sleep(sleep_time / 20.0)
 
 
 def get_cmd_args_dict():
@@ -893,15 +860,8 @@ def parse_cmd_args(argv_vals):
     return results
 
 
-def generate_guid():
-    """ Generate a GUID
-    """
-    return uuid.uuid4().hex
-
-
 def is_valid_experiment_key(experiment_key):
-    """ Validate an experiment_key; returns True or False
-    """
+    """Validate an experiment_key; returns True or False"""
     return (
         isinstance(experiment_key, str)
         and experiment_key.isalnum()
