@@ -49,8 +49,7 @@ except ImportError:  # everett version 0.9 for Python 2
     from everett.manager import ConfigIniEnv as UpstreamConfigIniEnv
 
     class ConfigIniEnv(UpstreamConfigIniEnv):
-        """ Backport of everett ConfigIniEnv in Python 3 that save the path when found
-        """
+        """Backport of everett ConfigIniEnv in Python 3 that save the path when found"""
 
         def __init__(self, possible_paths):
             super(ConfigIniEnv, self).__init__(possible_paths)
@@ -76,6 +75,16 @@ ADDITIONAL_STREAMER_UPLOAD_TIMEOUT = 3 * 60 * 60  # 3 hours
 
 DEFAULT_FILE_UPLOAD_READ_TIMEOUT = 900
 
+DEFAULT_ARTIFACT_DOWNLOAD_TIMEOUT = 3 * 60 * 60  # 3 hours
+
+DEFAULT_INITIAL_DATA_LOGGER_JOIN_TIMEOUT = 5 * 60
+
+DEFAULT_WS_JOIN_TIMEOUT = 10
+
+DEFAULT_PARAMETERS_UPDATE_INTERVAL_SECONDS = 60
+
+DEFAULT_OFFLINE_DATA_DIRECTORY = ".cometml-runs"
+
 
 def get_global_experiment():
     global experiment
@@ -682,7 +691,7 @@ CONFIG_MAP = {
     "comet.disable_auto_logging": {"type": int, "default": 0},
     "comet.api_key": {"type": str},
     "comet.rest_api_key": {"type": str},
-    "comet.offline_directory": {"type": str},
+    "comet.offline_directory": {"type": str, "default": DEFAULT_OFFLINE_DATA_DIRECTORY},
     "comet.git_directory": {"type": str},
     "comet.offline_sampling_size": {"type": int, "default": 15000},
     "comet.url_override": {"type": str, "default": "https://www.comet.ml/clientlib/"},
@@ -725,6 +734,10 @@ CONFIG_MAP = {
         "default": DEFAULT_FILE_UPLOAD_READ_TIMEOUT,
     },
     "comet.timeout.file_download": {"type": int, "default": 600},
+    "comet.timeout.artifact_download": {
+        "type": int,
+        "default": DEFAULT_ARTIFACT_DOWNLOAD_TIMEOUT,
+    },
     "comet.timeout.predictor": {"type": int, "default": 60},
     # HTTP Allow header
     "comet.allow_header.name": {"type": str},
@@ -815,6 +828,15 @@ def get_api_key(api_key, config):
     return final_api_key
 
 
+def discard_api_key(api_key):
+    # type: (str) -> None
+    """Discards the provided API key as invalid. After this method invocation the discarded key will not be masked in
+    the logger output.
+    """
+    if api_key is not None:
+        _get_comet_logging_config().discard_string_from_redact(api_key)
+
+
 def get_display_summary_level(display_summary_level, config):
     if display_summary_level is None:
         return config["comet.display_summary_level"]
@@ -829,7 +851,7 @@ def get_display_summary_level(display_summary_level, config):
 
 
 def get_ws_url(ws_server_from_backend, config):
-    """ Allow users to override the WS url from the backend using the usual
+    """Allow users to override the WS url from the backend using the usual
     config mechanism
     """
     ws_server_from_config = config["comet.ws_url_override"]
