@@ -437,7 +437,9 @@ class ConfusionMatrix(object):
         """
         if example is None:
             LOGGER.info(
-                CONFUSION_MATRIX_EXAMPLE_NONE, self.index_to_example_function, index,
+                CONFUSION_MATRIX_EXAMPLE_NONE,
+                self.index_to_example_function,
+                index,
             )
             return None
         elif isinstance(example, int):
