@@ -67,8 +67,7 @@ def stream_handler_set_stream(stream_handler_instance, stream):
 
 
 class FakeStd(object):
-    """ A fake Std file-like that sends every line to a handler.
-    """
+    """A fake Std file-like that sends every line to a handler."""
 
     def __init__(self, handler, original):
         self.__wrapped__ = original
@@ -206,7 +205,7 @@ def start_tee_process(*args, **kwargs):
 
 
 def _tee_process(pipe_out, control_pipe, real_out_fd, name, queue):
-    """ A loop that reads a pipe, write the content of the pipe into a given
+    """A loop that reads a pipe, write the content of the pipe into a given
     file descriptor and put it into a queue.
     Stops when there is some data in the control pipe.
     """
@@ -250,7 +249,7 @@ def _tee_process(pipe_out, control_pipe, real_out_fd, name, queue):
 
 
 class Unbuffered(object):
-    """ A custom sys.stdout and sys.stderr that force flush after every
+    """A custom sys.stdout and sys.stderr that force flush after every
     writing.
     """
 
@@ -274,7 +273,8 @@ class Unbuffered(object):
 
 try:
     import fcntl
-    from wurlitzer import libc, c_stdout_p, c_stderr_p, _default_encoding
+
+    from wurlitzer import _default_encoding, c_stderr_p, c_stdout_p, libc
 
     def _setup_fd(fd):
         """Common set-up code for initializing a (pipe) file descriptor"""
@@ -284,7 +284,7 @@ try:
         fcntl.fcntl(fd, fcntl.F_SETFL, flags | os.O_NONBLOCK)
 
     class PipeMultiprocessingStdWrapper(object):
-        """ This wrapper is very similar to the wurlitzer based-one that was
+        """This wrapper is very similar to the wurlitzer based-one that was
         used previously. The main difference with the wurlitzer one is that
         this logger uses subprocess in order to avoid having pipe consuming
         threads being blocked by the GIL held by the main thread running some
@@ -534,8 +534,7 @@ class StdLogger(object):
 
 
 def get_std_logger(wrapper_spec, streamer):
-    """ A factory that pass the right wrapper class to the StdLogger
-    """
+    """A factory that pass the right wrapper class to the StdLogger"""
     if wrapper_spec is None or wrapper_spec is False:
         return None
 
