@@ -94,8 +94,7 @@ def get_env_variables(blacklist, os_environ):
 
     def filter_env_variable(env_variable_name):
         # type: (str) -> bool
-        """ Filter environment variable names containing at least one of the blacklist pattern
-        """
+        """Filter environment variable names containing at least one of the blacklist pattern"""
         lowercase_env_variable_name = env_variable_name.lower()
         for pattern in lowercase_blacklist:
             if pattern in lowercase_env_variable_name:
@@ -150,7 +149,7 @@ def get_env_details_message(env_blacklist, include_env=False):
 
 def get_caller_file_path():
     # type: () -> Optional[Tuple[str, str]]
-    """ Returns the module name and file path from the first caller that isn't in the blacklisted
+    """Returns the module name and file path from the first caller that isn't in the blacklisted
     list. If none is found, returns None
     """
     # Don't get source code from these libraries:
