@@ -30,8 +30,7 @@ from .utils import format_version_number
 
 
 class CometException(Exception):
-    """ Base-class for all comet specific exceptions
-    """
+    """Base-class for all comet specific exceptions"""
 
 
 class NotParametrizedException(CometException):
@@ -148,8 +147,8 @@ class ExperimentAlreadyUploaded(CometException):
         return self.msg % self.experiment_id
 
 
-class FileIsTooBig(CometException):
-    msg = "File %r size %d is greater than the upload limit %d"
+class AssetIsTooBig(CometException):
+    msg = "Asset %r size %d is greater than the upload limit %d"
 
     def __init__(self, file_path, file_size, max_size):
         self.file_path = file_path
@@ -205,6 +204,21 @@ class ArtifactNotFound(CometException):
     pass
 
 
+class ArtifactAssetNotFound(CometException):
+    def __init__(self, logical_path, logged_artifact):
+        self.logical_path = logical_path
+        self.logged_artifact = logged_artifact
+
+    def __repr__(self):
+        return "No asset found for logical_path %r in %r" % (
+            self.logical_path,
+            self.logged_artifact,
+        )
+
+    def __str__(self):
+        return self.__repr__()
+
+
 class ArtifactDownloadException(CometException):
     pass
 
@@ -213,6 +227,23 @@ class ArtifactNotFinalException(CometException):
     pass
 
 
+class ArtifactConflictingAssetLogicalPath(CometException):
+    def __init__(self, existing_asset, conflicting_asset, common_logical_path):
+        # type (Any, str, Any) -> None
+        self.existing_asset = existing_asset
+        self.conflicting_asset = conflicting_asset
+        self.common_logical_path = common_logical_path
+
+    def __repr__(self):
+        return (
+            "Cannot add new asset (%r) with logical_path %r, an existing asset already exists with this logical_path (%r). To add this asset to this artifact you should use a new unique logical_path"
+            % (self.conflicting_asset, self.common_logical_path, self.existing_asset)
+        )
+
+    def __str__(self):
+        return self.__repr__()
+
+
 class ExperimentCleaningException(CometException):
     pass
 
