@@ -21,6 +21,7 @@ import logging
 import os
 import shutil
 import tempfile
+from collections import namedtuple
 
 import six
 
@@ -36,7 +37,7 @@ from ._typing import (
     ValidFilePath,
 )
 from .convert_utils import convert_pathlib_path
-from .exceptions import FileIsTooBig
+from .exceptions import AssetIsTooBig
 from .logging_messages import (
     LOG_AUDIO_TOO_BIG,
     LOG_FIGURE_TOO_BIG,
@@ -47,8 +48,10 @@ from .logging_messages import (
 from .messages import RemoteAssetMessage, UploadFileMessage, UploadInMemoryMessage
 from .utils import (
     data_to_fp,
+    generate_guid,
     get_file_extension,
     image_data_to_file_like_object,
+    parse_remote_uri,
     write_file_like_to_tmp_file,
     write_numpy_array_as_wav,
 )
@@ -95,7 +98,7 @@ def is_user_text(input):
 def check_max_file_size(file_path, max_upload_size, too_big_msg):
     # type: (str, int, str) -> int
     """Check if a file identified by its file path is bigger than the maximum
-    allowed upload size. Raises FileIsTooBig if the file is greater than the
+    allowed upload size. Raises AssetIsTooBig if the file is greater than the
     upload limit.
     """
 
@@ -103,7 +106,7 @@ def check_max_file_size(file_path, max_upload_size, too_big_msg):
     try:
         file_size = os.path.getsize(file_path)
         if file_size > max_upload_size:
-            raise FileIsTooBig(file_path, file_size, max_upload_size)
+            raise AssetIsTooBig(file_path, file_size, max_upload_size)
 
         return file_size
 
@@ -160,6 +163,27 @@ def save_plotly_figure(figure):
     return tmpfile.name
 
 
+def total_len(o):
+    if hasattr(o, "__len__"):
+        return len(o)
+
+    if hasattr(o, "getvalue"):
+        # e.g. BytesIO, cStringIO.StringIO
+        return len(o.getvalue())
+
+    if hasattr(o, "fileno"):
+        try:
+            fileno = o.fileno()
+        except OSError:
+            pass
+        else:
+            return os.fstat(fileno).st_size
+
+    raise NotImplementedError(
+        "Don't know how to compute total_len for %r[%r]", o, o.__class__
+    )
+
+
 class AssetUploadUserInput(object):
     def __init__(self, user_input):
         self.user_input = user_input
@@ -170,22 +194,38 @@ class FileUpload(AssetUploadUserInput):
         super(FileUpload, self).__init__(file_path)
 
         try:
-            self.file_size = os.path.getsize(file_path)
+            self.size = os.path.getsize(file_path)
         except OSError:
             LOGGER.debug("Error retrieving file size for %r", file_path)
-            self.file_size = 0
+            self.size = 0
 
 
 class MemoryFileUpload(AssetUploadUserInput):
-    pass
+    def __init__(self, user_input):
+        super(MemoryFileUpload, self).__init__(user_input)
+
+        try:
+            self.size = total_len(user_input)
+        except Exception:
+            LOGGER.debug("Error retrieving size for %r", user_input)
+            self.size = 0
 
 
 class UserTextFileUpload(AssetUploadUserInput):
-    pass
+    def __init__(self, user_input):
+        super(UserTextFileUpload, self).__init__(user_input)
+
+        try:
+            self.size = total_len(user_input)
+        except Exception:
+            LOGGER.debug("Error retrieving size for %r", user_input)
+            self.size = 0
 
 
 class ObjectToConvertFileUpload(AssetUploadUserInput):
-    pass
+    def __init__(self, user_input):
+        super(ObjectToConvertFileUpload, self).__init__(user_input)
+        self.size = 0
 
 
 def dispatch_user_file_upload(user_input):
@@ -212,7 +252,7 @@ class BaseUploadProcessor(object):
         self,
         user_input,  # type: Any
         upload_limit,  # type: int
-        url_params,  # type: Dict[str, Optional[Any]]
+        url_params,  # type: Optional[Dict[str, Optional[Any]]]
         metadata,  # type: Optional[Dict[str, str]]
         copy_to_tmp,  # type: bool
         error_message_identifier,  # type: Any
@@ -279,7 +319,7 @@ class BaseUploadProcessor(object):
             self.file_size = check_max_file_size(
                 user_filepath, self.upload_limit, self.TOO_BIG_MSG
             )
-        except FileIsTooBig as exc:
+        except AssetIsTooBig as exc:
             if self.error_message_identifier is None:
                 error_message_identifier = exc.file_path
             else:
@@ -323,6 +363,7 @@ class BaseUploadProcessor(object):
             self.UPLOAD_TYPE,
             self.url_params,
             self.metadata,
+            size=0,  # TODO: Replace by pre-processing
             clean=clean,
             critical=self.critical,
             on_asset_upload=self.on_asset_upload,
@@ -382,6 +423,7 @@ class BaseUploadProcessor(object):
             self.UPLOAD_TYPE,
             self.url_params,
             self.metadata,
+            size=0,  # TODO: Replace by pre-processing
             critical=self.critical,
             on_asset_upload=self.on_asset_upload,
             on_failed_asset_upload=self.on_failed_asset_upload,
@@ -413,6 +455,7 @@ class BaseUploadProcessor(object):
             self.UPLOAD_TYPE,
             self.url_params,
             self.metadata,
+            size=0,  # TODO: Replace by pre-processing
             critical=self.critical,
             on_asset_upload=self.on_asset_upload,
             on_failed_asset_upload=self.on_failed_asset_upload,
@@ -800,38 +843,350 @@ class GitPatchUploadProcessor(BaseUploadProcessor):
         return self._process_upload_by_filepath(upload_filepath)
 
 
-class RemoteAssetUploadProcessor(object):
-    def __init__(
-        self,
-        remote_uri,  # type: Any
-        upload_type,  # type: str
-        url_params,  # type: Dict[str, Optional[Any]]
-        metadata,  # type: Optional[Dict[str, str]]
-        critical,  # type: bool
-        on_asset_upload=None,
-        on_failed_asset_upload=None,
+class PreprocessedRemoteAsset(
+    namedtuple(
+        "_PreprocessedRemoteAsset",
+        [
+            "remote_uri",
+            "overwrite",
+            "upload_type",
+            "metadata",
+            "step",
+            "asset_id",
+            "logical_path",
+            "size",
+        ],
+    )
+):
+    __slots__ = ()
+
+    def to_message(
+        self, critical, on_asset_upload, on_failed_asset_upload, experiment_url_params
     ):
-        # type: (...) -> None
-        self.remote_uri = remote_uri
-        self.upload_type = upload_type
-        self.url_params = url_params
-        self.metadata = metadata
-        self.critical = critical
+        # type: (...) -> RemoteAssetMessage
+        url_params = {
+            "assetId": self.asset_id,
+            "fileName": self.logical_path,
+            "isRemote": True,
+            "overwrite": self.overwrite,
+            "step": self.step,
+        }
 
-        self.on_asset_upload = on_asset_upload
-        self.on_failed_asset_upload = on_failed_asset_upload
+        # If the asset type is more specific, include the
+        # asset type as "type" in query parameters:
+        if self.upload_type != "asset":
+            url_params["type"] = self.upload_type
 
-    def process(self):
-        # type: () -> RemoteAssetMessage
-        # Add the expected params in the URL
-        self.url_params.update({"isRemote": True})
+        url_params.update(experiment_url_params)
 
         return RemoteAssetMessage(
             self.remote_uri,
             self.upload_type,
-            self.url_params,
+            url_params,
             self.metadata,
-            self.critical,
-            on_asset_upload=self.on_asset_upload,
-            on_failed_asset_upload=self.on_failed_asset_upload,
+            size=self.size,
+            critical=critical,
+            on_asset_upload=on_asset_upload,
+            on_failed_asset_upload=on_failed_asset_upload,
         )
+
+
+def preprocess_remote_asset(
+    remote_uri,  # type: Any
+    logical_path,  # type: Optional[str]
+    overwrite,  # type: bool
+    upload_type,  # type: str
+    metadata,  # type: Optional[Dict[str, str]]
+    asset_id=None,  # type: Optional[str]
+    step=None,  # type: Optional[int]
+):
+    # type: (...) -> PreprocessedRemoteAsset
+
+    if asset_id is None:
+        asset_id = generate_guid()
+    asset_id = asset_id
+
+    if logical_path is None:
+        # Try to parse the URI to see if we can extract a useful file name
+        logical_path = parse_remote_uri(remote_uri)
+        if not logical_path:
+            LOGGER.info(
+                "Couldn't parse a file_name from URI %r, defaulting to 'remote'",
+                remote_uri,
+            )
+            logical_path = "remote"
+
+    try:
+        size = len(remote_uri)
+    except Exception:
+        LOGGER.debug("Couldn't compute size for remote uri %r", remote_uri)
+        size = 0
+
+    return PreprocessedRemoteAsset(
+        remote_uri,
+        overwrite,
+        upload_type,
+        validate_metadata(metadata),
+        step,
+        asset_id,
+        logical_path,
+        size,
+    )
+
+
+class PreprocessedFileAsset(
+    namedtuple(
+        "_PreprocessedFileAsset",
+        [
+            "local_path_or_data",
+            "upload_type",
+            "logical_path",
+            "metadata",
+            "overwrite",
+            "copy_to_tmp",
+            "step",
+            "asset_id",
+            "grouping_name",
+            "extension",
+            "size",
+        ],
+    )
+):
+    __slots__ = ()
+
+    def to_message(
+        self,
+        critical,
+        on_asset_upload,
+        on_failed_asset_upload,
+        clean,
+        experiment_url_params,
+    ):
+        # type: (...) -> UploadFileMessage
+        url_params = {
+            "assetId": self.asset_id,
+            "extension": self.extension,
+            "fileName": self.logical_path,
+            "overwrite": self.overwrite,
+        }
+
+        # If the asset type is more specific, include the
+        # asset type as "type" in query parameters:
+        if self.upload_type != "asset":
+            url_params["type"] = self.upload_type
+
+        if self.grouping_name is not None:
+            url_params["groupingName"] = self.grouping_name
+
+        url_params.update(experiment_url_params)
+
+        return UploadFileMessage(
+            self.local_path_or_data,
+            self.upload_type,
+            url_params,
+            self.metadata,
+            clean=clean,
+            critical=critical,
+            size=self.size,
+            on_asset_upload=on_asset_upload,
+            on_failed_asset_upload=on_failed_asset_upload,
+        )
+
+    def copy(self, new_local_path, new_copy_to_tmp):
+        return self.__class__(
+            local_path_or_data=new_local_path,
+            upload_type=self.upload_type,
+            logical_path=self.logical_path,
+            metadata=self.metadata,
+            overwrite=self.overwrite,
+            copy_to_tmp=new_copy_to_tmp,
+            step=self.step,
+            asset_id=self.asset_id,
+            grouping_name=self.grouping_name,
+            extension=self.extension,
+            size=self.size,
+        )
+
+
+def preprocess_asset_file(
+    dispatched,  # type: FileUpload
+    upload_type,  # type: str
+    file_name,  # type: Optional[str]
+    metadata,  # type: Optional[Dict[str, str]]
+    overwrite,  # type: bool
+    copy_to_tmp,  # type: bool
+    step=None,  # type: Optional[int]
+    grouping_name=None,  # type: Optional[str]
+    asset_id=None,  # type: Optional[Any]
+):
+    # type: (...) -> PreprocessedFileAsset
+    upload_filepath = dispatched.user_input
+
+    if asset_id is None:
+        asset_id = generate_guid()
+    asset_id = asset_id
+
+    if file_name is None:
+        file_name = os.path.basename(upload_filepath)
+
+    extension = get_file_extension(upload_filepath)
+
+    size = dispatched.size
+
+    return PreprocessedFileAsset(
+        upload_filepath,
+        upload_type,
+        file_name,
+        validate_metadata(metadata),
+        overwrite,
+        copy_to_tmp,
+        step,
+        asset_id,
+        grouping_name,
+        extension,
+        size,
+    )
+
+
+class PreprocessedMemoryFileAsset(
+    namedtuple(
+        "_PreprocessedMemoryFileAsset",
+        [
+            "local_path_or_data",
+            "upload_type",
+            "logical_path",
+            "metadata",
+            "overwrite",
+            "copy_to_tmp",
+            "step",
+            "asset_id",
+            "grouping_name",
+            "extension",
+            "size",
+        ],
+    )
+):
+    __slots__ = ()
+
+    def to_message(
+        self,
+        critical,
+        on_asset_upload,
+        on_failed_asset_upload,
+        experiment_url_params,
+        clean,
+    ):
+        # type: (...) -> UploadInMemoryMessage
+
+        # TODO: Clean is ignored but kept to keep the to_message API consistent with
+        # PreprocessedFileAsset.to_message
+        url_params = {
+            "assetId": self.asset_id,
+            "extension": self.extension,
+            "fileName": self.logical_path,
+            "overwrite": self.overwrite,
+        }
+
+        # If the asset type is more specific, include the
+        # asset type as "type" in query parameters:
+        if self.upload_type != "asset":
+            url_params["type"] = self.upload_type
+
+        if self.grouping_name is not None:
+            url_params["groupingName"] = self.grouping_name
+
+        url_params.update(experiment_url_params)
+
+        return UploadInMemoryMessage(
+            self.local_path_or_data,
+            self.upload_type,
+            url_params,
+            self.metadata,
+            size=self.size,
+            critical=critical,
+            on_asset_upload=on_asset_upload,
+            on_failed_asset_upload=on_failed_asset_upload,
+        )
+
+    def to_preprocessed_file_asset(self, new_local_path, new_copy_to_tmp):
+        return PreprocessedFileAsset(
+            local_path_or_data=new_local_path,
+            upload_type=self.upload_type,
+            logical_path=self.logical_path,
+            metadata=self.metadata,
+            overwrite=self.overwrite,
+            copy_to_tmp=new_copy_to_tmp,
+            step=self.step,
+            asset_id=self.asset_id,
+            grouping_name=self.grouping_name,
+            extension=self.extension,
+            size=self.size,
+        )
+
+
+def preprocess_asset_memory_file(
+    dispatched,  # type: MemoryFileUpload
+    upload_type,  # type: str
+    file_name,  # type: Optional[str]
+    metadata,  # type: Optional[Dict[str, str]]
+    overwrite,  # type: bool
+    copy_to_tmp,  # type: bool
+    step=None,  # type: Optional[int]
+    grouping_name=None,  # type: Optional[str]
+    asset_id=None,  # type: Optional[Any]
+):
+    # type: (...) -> PreprocessedMemoryFileAsset
+    file_like = dispatched.user_input
+
+    if asset_id is None:
+        asset_id = generate_guid()
+    asset_id = asset_id
+
+    extension = get_file_extension(file_name)
+
+    size = dispatched.size
+
+    # XXX: Previously when no file_name was given and copy_to_tmp was set to True, we would uses the
+    # temporary file basename, given random file name. Now we raises an Exception in all cases
+    if file_name is None:
+        raise TypeError("file_name shouldn't be None")
+
+    return PreprocessedMemoryFileAsset(
+        file_like,
+        upload_type,
+        file_name,
+        validate_metadata(metadata),
+        overwrite,
+        copy_to_tmp,
+        step,
+        asset_id,
+        grouping_name,
+        extension,
+        size,
+    )
+
+
+PreprocessedAsset = Union[
+    PreprocessedFileAsset, PreprocessedMemoryFileAsset, PreprocessedRemoteAsset
+]
+
+
+def handle_in_memory_file_upload(tmp_dir, upload_filepath):
+    # type: (str, ValidFilePath) -> Union[None, ValidFilePath, TemporaryFilePath]
+    # If we cannot remove the uploaded file or need the file content will
+    # be frozen to the time the upload call is made, pass copy_to_tmp with
+    # True value
+    if not isinstance(upload_filepath, TemporaryFilePath):
+        tmpfile = tempfile.NamedTemporaryFile(delete=False, dir=tmp_dir)
+        tmpfile.close()
+        LOGGER.debug(
+            "Copying %s to %s because of copy_to_tmp", upload_filepath, tmpfile.name
+        )
+        try:
+            shutil.copyfile(upload_filepath, tmpfile.name)
+        except (OSError, IOError):
+            LOGGER.error(UPLOAD_FILE_OS_ERROR, upload_filepath, exc_info=True)
+            return None
+        upload_filepath = TemporaryFilePath(tmpfile.name)
+
+    return upload_filepath
