@@ -14,8 +14,9 @@
 
 import logging
 
-from .._logging import GET_CALLBACK_FAILURE, check_module
 from .._typing import Any, Dict
+from ..logging_messages import GET_CALLBACK_FAILURE
+from ..monkey_patching import check_module
 from . import tensorboard_logger
 
 LOGGER = logging.getLogger(__name__)
