@@ -25,25 +25,37 @@ LOGGER = logging.getLogger(__name__)
 
 def _get_loss(loss_backward, experiment):
     # type: (Any, BaseExperiment) -> Any
-    """ Returns the right loss tensor based on the loss tensor object where
+    """Returns the right loss tensor based on the loss tensor object where
     backward has been called. The right loss might be the unscaled one when
     using APEX or Pytorch 1.6 AMP
     """
-    return experiment._localstorage.pytorch["amp_loss_mapping"].get(
-        id(loss_backward), loss_backward
-    )
+    if experiment.alive:
+        return experiment._localstorage.pytorch["amp_loss_mapping"].get(
+            id(loss_backward), loss_backward
+        )
+    else:
+        LOGGER.warning("Attempting to get amp_loss_mapping for closed experiment")
+        return None
 
 
 def _add_amp_loss_mapping(scaled_loss, unscaled_loss, experiment):
     # type: (int, Any, BaseExperiment) -> None
     # First argument is the scaled_loss id
-    experiment._localstorage.pytorch["amp_loss_mapping"][scaled_loss] = unscaled_loss
+    if experiment.alive:
+        experiment._localstorage.pytorch["amp_loss_mapping"][
+            scaled_loss
+        ] = unscaled_loss
+    else:
+        LOGGER.warning("Attempting to update amp_loss_mapping for closed experiment")
 
 
 def _clean_amp_loss_mapping(scaled_loss, experiment):
     # type: (int, BaseExperiment) -> None
     # First argument is the scaled_loss id
-    experiment._localstorage.pytorch["amp_loss_mapping"].pop(scaled_loss, None)
+    if experiment.alive:
+        experiment._localstorage.pytorch["amp_loss_mapping"].pop(scaled_loss, None)
+    else:
+        LOGGER.warning("Attempting to clean amp_loss_mapping for closed experiment")
 
 
 def tensor_backward(experiment, original, result, *args, **kwargs):
@@ -67,7 +79,10 @@ def tensor_backward(experiment, original, result, *args, **kwargs):
                 if len(loss.data.shape) == 0:
                     metric = loss.data.item()
                     experiment._log_metric(
-                        "loss", metric, step=experiment.curr_step, framework="pytorch",
+                        "loss",
+                        metric,
+                        step=experiment.curr_step,
+                        framework="pytorch",
                     )
                 else:
                     experiment._log_metric(
