@@ -252,13 +252,19 @@ def patch(module_finder):
         summary_histogram_3d_logger,
     )
     module_finder.register_after(
-        "tensorboard.plugins.text.summary_v2", "text", summary_text_logger,
+        "tensorboard.plugins.text.summary_v2",
+        "text",
+        summary_text_logger,
     )
     module_finder.register_after(
-        "tensorboard.plugins.audio.summary_v2", "audio", summary_audio_logger,
+        "tensorboard.plugins.audio.summary_v2",
+        "audio",
+        summary_audio_logger,
     )
     module_finder.register_after(
-        "tensorboard.plugins.image.summary_v2", "image", summary_image_logger,
+        "tensorboard.plugins.image.summary_v2",
+        "image",
+        summary_image_logger,
     )
 
 
