@@ -50,6 +50,7 @@ optimizers_hyper_params = {
         "_initial_accumulator_value",
         "_l1_regularization_strength",
         "_l2_regularization_strength",
+        "_l2_shrinkage_regularization_strength",
     ],
     "AdagradOptimizer": ["_learning_rate", "_initial_accumulator_value"],
 }
@@ -117,7 +118,18 @@ OPTIMIZER_V2 = [
     ("tensorflow.python.keras.optimizer_v2.gradient_descent", "SGD.__init__"),
     ("tensorflow.python.keras.optimizer_v2.nadam", "Nadam.__init__"),
     ("tensorflow.python.keras.optimizer_v2.rmsprop", "RMSprop.__init__"),
-    ("tensorflow.python.keras.optimizer_v2.rmsprop", "RMSProp.__init__"),
+]
+
+# TF 2.6.0 have moved the optimizers code to keras
+OPTIMIZER_V2_6 = [
+    ("keras.optimizer_v2.adam", "Adam.__init__"),
+    ("keras.optimizer_v2.adadelta", "Adadelta.__init__"),
+    ("keras.optimizer_v2.adagrad", "Adagrad.__init__"),
+    ("keras.optimizer_v2.adamax", "Adamax.__init__"),
+    ("keras.optimizer_v2.ftrl", "Ftrl.__init__"),
+    ("keras.optimizer_v2.gradient_descent", "SGD.__init__"),
+    ("keras.optimizer_v2.nadam", "Nadam.__init__"),
+    ("keras.optimizer_v2.rmsprop", "RMSprop.__init__"),
 ]
 
 ESTIMATOR_V1 = [
@@ -312,6 +324,10 @@ def patch(module_finder):
     for module, object_name in OPTIMIZER_V2:
         module_finder.register_after(module, object_name, optimizer_logger_v2)
 
+    # Register the v2.6.x methods
+    for module, object_name in OPTIMIZER_V2_6:
+        module_finder.register_after(module, object_name, optimizer_logger_v2)
+
     # Estimators:
     for module, object_name in ESTIMATOR_V2:
         module_finder.register_after(module, object_name, estimator_logger)
