@@ -16,9 +16,9 @@ import copy
 import logging
 from functools import partial
 
-from .._logging import check_module
 from .._typing import Any, Dict, Generator, List, Set, Tuple
 from ..experiment import BaseExperiment
+from ..monkey_patching import check_module
 
 LOGGER = logging.getLogger(__name__)
 
