@@ -34,6 +34,8 @@ INVALID_API_KEY = (
     "Your experiment would not be logged" + GO_TO_DOCS_MSG
 )
 
+WAITING_DATA_UPLOADED = "Uploading metrics, params, and assets to Comet before program termination (may take several seconds)"
+
 INVALID_REST_API_KEY = (
     "The given REST API key %s is invalid, please check it against the dashboard. "
 )
@@ -90,16 +92,21 @@ OFFLINE_SENDER_STARTS = "Starting the upload of the experiment"
 
 OFFLINE_SENDER_ENDS = "The offline experiment has been uploaded on comet.ml %s\n"
 
+OFFLINE_DATA_DIR_DEFAULT_WARNING = (
+    "Using %r path as offline directory. Pass 'offline_directory' parameter into constructor "
+    "or set the 'COMET_OFFLINE_DIRECTORY' environment variable to manually choose where to store offline "
+    "experiment archives."
+)
+
+OFFLINE_DATA_DIR_FAILED_WARNING = (
+    "Failed to use offline data directory %r, the experiment will be saved in the temporary directory %r instead.\n"
+    "Reason: %s"
+)
+
 ADD_TAGS_ERROR = "Failed to add tag(s) %s to the experiment\n"
 
 ADD_SYMLINK_ERROR = "Failed to create symlink to project:%s for experiment\n"
 
-OFFLINE_EXPERIMENT_TEMPORARY_DIRECTORY = (
-    "The offline directory %r was not usable.\n"
-    "Reason: %r\n"
-    "Saving the experiment in directory %s instead."
-)
-
 OFFLINE_EXPERIMENT_INVALID_WS_MSG = "An invalid message has been detected"
 
 OFFLINE_EXPERIMENT_INVALID_UPLOAD_MSG = "An invalid upload message has been detected"
@@ -108,10 +115,29 @@ EXPERIMENT_INVALID_STEP = "Passed step value %r is not a number, ignoring it"
 
 EXPERIMENT_INVALID_EPOCH = "Passed epoch value %r is not a number, ignoring it"
 
+EXPERIMENT_INITIAL_DATA_LOGGER_INCOMPLETE = (
+    "Not all initial data has been logged for experiment %s, call Experiment.end() "
+    "to ensure that all data to have been logged"
+)
+
+EXPERIMENT_GET_PARAMETER_SHORT_NAME_DEPRECATION = (
+    "Getting a parameter without its full name is deprecated and will be removed in the future, "
+    "potential full-names for the '%s' parameter are %s"
+)
+
 STREAMER_WAIT_FOR_FINISH_FAILED = (
     "Comet failed to send all the data back (%d messages and %d uploads)"
 )
 
+STREAMER_CLOSED_PUT_MESSAGE_FAILED = (
+    "Attempting to put message into closed streamer: message ignored! "
+    "You need to call Experiment.end() to have everything logged properly."
+)
+
+STREAMER_FAILED_TO_PROCESS_ALL_MESSAGES = (
+    "Failed to send all messages, metrics and output will likely be incomplete."
+)
+
 REGISTER_RPC_FAILED = "Failed to register callback named %r"
 
 WS_SSL_ERROR_MSG = (
@@ -158,12 +184,14 @@ MLFLOW_OFFLINE_EXPERIMENT_FALLBACK = "No Comet API Key was found, creating an Of
 
 MLFLOW_RESUMED_RUN = "Resumed MLFlow run are tracked as new Comet.ml Experiments"
 
-ONLINE_EXPERIMENT_THROTTLED = "Experiment has been throttled, some data will be missing"
+ONLINE_EXPERIMENT_THROTTLED = "Experiment has been throttled. Some data (like experiment metrics) might be missing. Mission-critical data, like Artifacts, are never throttled."
 
 TF_KERAS_FALLBACK_FAILED = (
     "Keras is not available; attempted to fallback to tensorflow.keras but still failed"
 )
 
+TF_KERAS_CALLBACK_WARNING_CLOSED_EXPERIMENT = "A Keras callback is trying to report to a un-alive experiment object (it was either ended or the creation failed) from the %s callback method, data will not be logged."
+
 CONFUSION_MATRIX_ERROR = "Error creating confusion matrix: %s; ignoring"
 
 CONFUSION_MATRIX_GENERAL_ERROR = "Error logging confusion matrix; ignoring"
@@ -286,3 +314,51 @@ ARTIFACT_PREVIEW_FEATURE_FLAG = (
 LOG_ARTIFACT_IN_PROGESS_MESSAGE = (
     "Still scheduling the upload of %d assets, remaining size %s"
 )
+
+FILE_UPLOADS_PROMPT = (
+    "Waiting for completion of the file uploads (may take several seconds)"
+)
+
+FILE_UPLOAD_MANAGER_MONITOR_FIRST_MESSAGE = (
+    "Still uploading %d file(s), remaining %s/%s"
+)
+
+FILE_UPLOAD_MANAGER_MONITOR_WAITING_BACKEND_ANSWER = (
+    "All files uploaded, waiting for confirmation they have been all received"
+)
+
+FILE_UPLOAD_MANAGER_MONITOR_PROGRESSION = (
+    "Still uploading %d file(s), remaining %s/%s, Throughput %s/s, ETA ~%ss"
+)
+
+FILE_UPLOAD_MANAGER_MONITOR_PROGRESSION_UNKOWN_ETA = (
+    "Still uploading %d file(s), remaining %s/%s, Throughput %s/s, ETA unknown"
+)
+
+ARTIFACT_DOWNLOAD_START_MESSAGE = (
+    "Artifact '%s/%s:%s' download has been started asynchronously"
+)
+
+ARTIFACT_DOWNLOAD_FINISHED = "Artifact '%s/%s:%s' has been successfully downloaded"
+
+ARTIFACT_ASSET_DOWNLOAD_FAILED = "Asset %r of the artifact '%s/%s:%s' download failed"
+
+FILE_DOWNLOAD_MANAGER_COMPLETED = "All files downloaded"
+
+FILE_DOWNLOAD_MANAGER_MONITOR_FIRST_MESSAGE = (
+    "Still downloading %d file(s), remaining %s/%s"
+)
+
+FILE_DOWNLOAD_MANAGER_MONITOR_PROGRESSION_UNKNOWN_ETA = (
+    "Still downloading %d file(s), remaining %s/%s, Throughput %s/s, ETA unknown"
+)
+
+FILE_DOWNLOAD_MANAGER_MONITOR_PROGRESSION = (
+    "Still downloading %d file(s), remaining %s/%s, Throughput %s/s, ETA ~%ss"
+)
+
+INITIAL_DATA_LOGGER_FLUSHING_EXPERIMENT_INTERRUPTED_BY_USER = "Experiment was interrupted by user while waiting for the initial data logger to be flushed."
+
+INITIAL_DATA_LOGGER_FLUSHING_FAILED = (
+    "Unexpected failure while waiting for the initial data logger to be flushed."
+)
