@@ -41,8 +41,7 @@ class BaseMessage(object):
 
 
 class CloseMessage(BaseMessage):
-    """ A special messag indicating Streamer to ends and exit
-    """
+    """A special messag indicating Streamer to ends and exit"""
 
     pass
 
@@ -53,7 +52,6 @@ class Message(BaseMessage):
     """
 
     def __init__(self, context=None):
-        self.context = context
         self.local_timestamp = local_timestamp()
 
         # The following attributes are optional
@@ -163,6 +161,7 @@ class UploadFileMessage(BaseMessage):
         upload_type,  # type: str
         additional_params,  # type: Dict[str, Optional[Any]]
         metadata,  # type: Optional[Any]
+        size,  # type: int
         clean=True,  # type: bool
         critical=False,  # type: bool
         on_asset_upload=None,  # type: Optional[Callable]
@@ -176,6 +175,7 @@ class UploadFileMessage(BaseMessage):
         self.additional_params = additional_params
         self.metadata = metadata
         self.clean = clean
+        self._size = size
         self._critical = critical
         self._on_asset_upload = on_asset_upload
         self._on_failed_asset_upload = on_failed_asset_upload
@@ -204,6 +204,7 @@ class UploadInMemoryMessage(BaseMessage):
         upload_type,  # type: str
         additional_params,  # type: Dict[str, Optional[Any]]
         metadata,  # type: Optional[Any]
+        size,  # type: int
         critical=False,  # type: bool
         on_asset_upload=None,  # type: Optional[Callable]
         on_failed_asset_upload=None,  # type: Optional[Callable]
@@ -215,6 +216,7 @@ class UploadInMemoryMessage(BaseMessage):
         self.upload_type = upload_type
         self.additional_params = additional_params
         self.metadata = metadata
+        self._size = size
         self._critical = critical
         self._on_asset_upload = on_asset_upload
         self._on_failed_asset_upload = on_failed_asset_upload
@@ -243,6 +245,7 @@ class RemoteAssetMessage(BaseMessage):
         upload_type,  # type: str
         additional_params,  # type: Dict[str, Optional[Any]]
         metadata,  # type: Optional[Dict[str, str]]
+        size,  # type: int
         critical=False,  # type: bool
         on_asset_upload=None,
         on_failed_asset_upload=None,
@@ -252,6 +255,7 @@ class RemoteAssetMessage(BaseMessage):
         self.upload_type = upload_type
         self.additional_params = additional_params
         self.metadata = metadata
+        self._size = size
         self._critical = critical
         self._on_asset_upload = on_asset_upload
         self._on_failed_asset_upload = on_failed_asset_upload
