@@ -26,15 +26,13 @@ from ._typing import Dict, List, Optional
 
 class ReservoirSampler(object):
     def __init__(self, num_samples):
-        """Create a new sampler with a certain reservoir size.
-        """
+        """Create a new sampler with a certain reservoir size."""
         self.num_samples = num_samples
         self.num_items_seen = 0
         self.reservoir = []  # type: List[float]
 
     def sample(self, item):
-        """ Sample an item and store in the reservoir if needed.
-        """
+        """Sample an item and store in the reservoir if needed."""
         num_samples = self.num_samples
 
         if len(self.reservoir) < num_samples:
@@ -49,8 +47,7 @@ class ReservoirSampler(object):
         self.num_items_seen += 1
 
     def get_samples(self):
-        """Get samples collected in the reservoir.
-        """
+        """Get samples collected in the reservoir."""
         return self.reservoir
 
 
@@ -168,7 +165,7 @@ class MetricsSampler(object):
         return self.pipelines[metric_name]
 
     def sample_metric(self, metric):
-        """ Sample the next element of the metric stream for the metric names.
+        """Sample the next element of the metric stream for the metric names.
         There are few rules about which messages we should keep at all costs:
         - The first message of the stream
         - The last message of the stream
