@@ -21,6 +21,7 @@ This module contains the code related to offline feature
 import io
 import json
 import logging
+import os
 import os.path
 import shutil
 import tempfile
@@ -37,11 +38,19 @@ from ._reporting import (
     OFFLINE_INVALID_WS_MSG,
 )
 from ._typing import Optional
-from .comet import OfflineStreamer, format_url, generate_guid, is_valid_experiment_key
+from .comet import OfflineStreamer, format_url, is_valid_experiment_key
 from .compat_utils import json_dump
-from .config import get_api_key, get_config, get_previous_experiment
+from .config import (
+    ADDITIONAL_STREAMER_UPLOAD_TIMEOUT,
+    DEFAULT_OFFLINE_DATA_DIRECTORY,
+    DEFAULT_WS_JOIN_TIMEOUT,
+    get_api_key,
+    get_config,
+    get_previous_experiment,
+)
 from .connection import (
     FileUploadManager,
+    FileUploadManagerMonitor,
     RestServerConnection,
     WebSocketConnection,
     format_messages_for_ws,
@@ -58,15 +67,18 @@ from .experiment import BaseExperiment
 from .feature_toggles import FeatureToggles
 from .logging_messages import (
     CLOUD_DETAILS_MSG_SENDING_ERROR,
+    FILE_UPLOADS_PROMPT,
     MODEL_GRAPH_MSG_SENDING_ERROR,
+    OFFLINE_DATA_DIR_DEFAULT_WARNING,
+    OFFLINE_DATA_DIR_FAILED_WARNING,
     OFFLINE_EXPERIMENT_ALREADY_UPLOADED,
     OFFLINE_EXPERIMENT_END,
     OFFLINE_EXPERIMENT_INVALID_UPLOAD_MSG,
     OFFLINE_EXPERIMENT_INVALID_WS_MSG,
-    OFFLINE_EXPERIMENT_TEMPORARY_DIRECTORY,
     OFFLINE_SENDER_ENDS,
     OFFLINE_SENDER_STARTS,
     OS_PACKAGE_MSG_SENDING_ERROR,
+    WAITING_DATA_UPLOADED,
 )
 from .metrics import MetricsSampler
 from .schemas import (
@@ -79,7 +91,7 @@ from .schemas import (
     get_upload_msg_validator,
     get_ws_msg_validator,
 )
-from .utils import local_timestamp
+from .utils import generate_guid, local_timestamp, wait_for_done
 
 LOGGER = logging.getLogger(__name__)
 
@@ -174,14 +186,6 @@ class OfflineExperiment(BaseExperiment):
             api_key, self.config
         )  # optional, except for on-line operations
 
-        if offline_directory is None:
-            offline_directory = self.config["comet.offline_directory"]
-
-        if offline_directory is None:
-            raise ValueError("OfflineExperiment needs an offline directory")
-
-        self.offline_directory = offline_directory
-
         # Start and ends time
         self.start_time = None
         self.stop_time = None
@@ -217,13 +221,15 @@ class OfflineExperiment(BaseExperiment):
 
         if not self.disabled:
             # Check that the offline directory is usable
-            try:
-                # Try to create the archive now
-                zipfile = self._get_offline_archive(self.offline_directory, self.id)
-                # Close the file handle, it will be reopened later
-                zipfile.close()
-            except (OSError, IOError) as exc:
-                raise InvalidOfflineDirectory(self.offline_directory, str(exc))
+            self.offline_directory, default_dir = self._get_offline_data_dir_path(
+                offline_directory
+            )
+            # Try to create ZIP file for the experiment
+            zip_file, self.offline_directory = self._create_offline_archive(
+                fallback_to_temp=default_dir
+            )
+            # Close the file handle, it will be reopened later
+            zip_file.close()
 
         if self.disabled is not True:
             if api_key is not None:
@@ -242,19 +248,81 @@ class OfflineExperiment(BaseExperiment):
             if self.alive is True:
                 self._report(event_name=EXPERIMENT_CREATED)
 
-    def display(self, *args, **kwargs):
-        """ Do nothing
+    def _create_offline_archive(self, fallback_to_temp=True):
+        # type: (bool) -> (ZipFile, str)
+        """Attempts to create ZIP file in the offline_directory. If attempt failed it will try to repeat in the
+        temporary directory if fallback_to_temp is set. Otherwise, InvalidOfflineDirectory raised.
+        Args:
+            fallback_to_temp:
+                The flag to indicate whether automatic recovery should be performed in case if failed to create archive
+                in the current offline_directory.
+        Returns:
+            The ZipFile for offline archive and the path to the offline directory where archive was created
         """
+        try:
+            # create parent directory
+            if not os.path.exists(self.offline_directory):
+                os.mkdir(self.offline_directory, 0o700)
+
+            # Try to create the archive now
+            return (
+                _create_zip_file(self.offline_directory, self.id),
+                self.offline_directory,
+            )
+        except (OSError, IOError) as exc:
+            if fallback_to_temp:
+                # failed - use temporary directory instead
+                offline_dir = tempfile.mkdtemp()
+                LOGGER.warning(
+                    OFFLINE_DATA_DIR_FAILED_WARNING,
+                    os.path.abspath(self.offline_directory),
+                    os.path.abspath(offline_dir),
+                    str(exc),
+                    exc_info=True,
+                )
+                return _create_zip_file(offline_dir, self.id), offline_dir
+            else:
+                raise InvalidOfflineDirectory(self.offline_directory, str(exc))
+
+    def _get_offline_data_dir_path(self, offline_directory):
+        # type: (Optional[str]) -> (str, bool)
+        """looks for the offline data directory path. If offline_directory parameter is not None it will be returned.
+        Otherwise, the experiment configuration settings will be used. Finally, if later failed the default data
+        directory will be used.
+        Args:
+            offline_directory:
+                The provided by user path to the offline directory or None if not set.
+        Returns:
+            The calculated path to the offline directory and flag to indicate if default data directory suffix was used.
+        """
+        if offline_directory is None:
+            offline_directory = self.config["comet.offline_directory"]
+
+        if (
+            offline_directory is not None
+            and offline_directory != DEFAULT_OFFLINE_DATA_DIRECTORY
+        ):
+            return offline_directory, False
+
+        # use DEFAULT_OFFLINE_DATA_DIRECTORY in the CWD
+        offline_directory = join(os.getcwd(), DEFAULT_OFFLINE_DATA_DIRECTORY)
+
+        # display warning to the user that offline directory was chosen with default path
+        LOGGER.info(OFFLINE_DATA_DIR_DEFAULT_WARNING, offline_directory)
+
+        return offline_directory, True
+
+    def display(self, *args, **kwargs):
+        """Do nothing"""
         pass
 
     def display_project(self, *args, **kwargs):
-        """ Do nothing
-        """
+        """Do nothing"""
         pass
 
-    def _start(self):
+    def _start(self, **kwargs):
         self.start_time = local_timestamp()
-        super(OfflineExperiment, self)._start()
+        super(OfflineExperiment, self)._start(**kwargs)
         self.log_other("offline_experiment", True)
 
     def _write_experiment_meta_file(self):
@@ -271,17 +339,6 @@ class OfflineExperiment(BaseExperiment):
         with open(meta_file_path, "wb") as f:
             json_dump(meta, f)
 
-    def _get_offline_archive(self, directory, name):
-        # ZIP the saved information
-        mode = 0o700
-        try:
-            os.mkdir(self.offline_directory, mode)
-        except os.error:
-            pass
-
-        file_path = self._offline_zip_path(directory, name)
-        return ZipFile(file_path, "w", allowZip64=True)
-
     def _mark_as_ended(self):
         if not self.alive:
             LOGGER.debug("Skipping creating the offline archive as we are not alive")
@@ -292,38 +349,28 @@ class OfflineExperiment(BaseExperiment):
 
         self._write_experiment_meta_file()
 
-        try:
-            zipfile = self._get_offline_archive(self.offline_directory, self.id)
-        except (OSError, IOError) as exc:
-            # Use a temporary directory if we came so far to not lose the information
-            old_dir = self.offline_directory
-            self.offline_directory = tempfile.mkdtemp()
-            zipfile = self._get_offline_archive(self.offline_directory, self.id)
-            LOGGER.warning(
-                OFFLINE_EXPERIMENT_TEMPORARY_DIRECTORY,
-                old_dir,
-                str(exc),
-                self.offline_directory,
-            )
+        zip_file, self.offline_directory = self._create_offline_archive(
+            fallback_to_temp=True
+        )
 
         for file in os.listdir(self.tmpdir):
-            zipfile.write(os.path.join(self.tmpdir, file), file)
+            zip_file.write(os.path.join(self.tmpdir, file), file)
 
-        zipfile.close()
+        zip_file.close()
 
         # Clean the tmpdir to avoid filling up the disk
         try:
             shutil.rmtree(self.tmpdir)
         except OSError:
             # We made our best effort to clean ourselves
-            msg = "Error cleaning offline experiment tmpdir %r"
-            LOGGER.debug(msg, self.tmpdir, exc_info=True)
+            LOGGER.debug(
+                "Error cleaning offline experiment tmpdir: %r",
+                self.tmpdir,
+                exc_info=True,
+            )
 
         # Display the full command to upload the offline experiment
-        LOGGER.info(OFFLINE_EXPERIMENT_END, zipfile.filename)
-
-    def _offline_zip_path(self, directory, name):
-        return os.path.join(directory, "%s.zip" % name)
+        LOGGER.info(OFFLINE_EXPERIMENT_END, zip_file.filename)
 
     def _setup_streamer(self):
         """
@@ -479,6 +526,7 @@ class OfflineSender(object):
         force_reupload=False,
         display_level="info",
         validation_error=False,
+        file_upload_waiting_timeout=ADDITIONAL_STREAMER_UPLOAD_TIMEOUT,
     ):
         self.config = get_config()
         self.api_key = api_key
@@ -514,6 +562,7 @@ class OfflineSender(object):
         self.file_upload_manager = FileUploadManager(
             self.config.get_int(None, "comet.internal.file_upload_worker_ratio")
         )
+        self.file_upload_waiting_timeout = file_upload_waiting_timeout
 
         self.validation_error = validation_error
 
@@ -691,16 +740,25 @@ class OfflineSender(object):
 
         additional_params = message["additional_params"] or {}
         additional_params["runId"] = self.run_id
+        # Temporary fix to ensure integers:
+        if "step" in additional_params and additional_params["step"] is not None:
+            additional_params["step"] = int(additional_params["step"])
+        if "epoch" in additional_params and additional_params["epoch"] is not None:
+            additional_params["epoch"] = int(additional_params["epoch"])
+
+        file_path = join(self.offline_dir, message["file_path"])
+        file_size = os.path.getsize(file_path)
 
         self.file_upload_manager.upload_file_thread(
             project_id=self.project_id,
             experiment_id=self.experiment_id,
-            file_path=join(self.offline_dir, message["file_path"]),
+            file_path=file_path,
             upload_endpoint=url,
             api_key=self.api_key,
             additional_params=additional_params,
             clean=True,
             timeout=self.config.get_int(None, "comet.timeout.file_upload"),
+            estimated_size=file_size,
         )
         LOGGER.debug("Processing uploading message done")
 
@@ -722,6 +780,11 @@ class OfflineSender(object):
 
         additional_params = message["additional_params"] or {}
         additional_params["runId"] = self.run_id
+        # Temporary fix to ensure integers:
+        if "step" in additional_params and additional_params["step"] is not None:
+            additional_params["step"] = int(additional_params["step"])
+        if "epoch" in additional_params and additional_params["epoch"] is not None:
+            additional_params["epoch"] = int(additional_params["epoch"])
 
         self.file_upload_manager.upload_remote_asset_thread(
             project_id=self.project_id,
@@ -886,9 +949,13 @@ class OfflineSender(object):
         return ""
 
     def close(self):
+        LOGGER.info(WAITING_DATA_UPLOADED)
         if self.ws_connection is not None:
             self.ws_connection.close()
-            ws_cleaned = self.ws_connection.wait_for_finish(timeout=None)
+            # Use 3 times the default timeout as they might be a lot of backend messages to process
+            ws_cleaned = self.ws_connection.wait_for_finish(
+                timeout=DEFAULT_WS_JOIN_TIMEOUT * 3
+            )
 
             if not ws_cleaned:
                 LOGGER.error(
@@ -899,6 +966,18 @@ class OfflineSender(object):
             self.rest_api_client.close()
 
         self.file_upload_manager.close()
+        # Finish remained uploads and display upload progress
+        if not self.file_upload_manager.all_done():
+            monitor = FileUploadManagerMonitor(self.file_upload_manager)
+            LOGGER.info(FILE_UPLOADS_PROMPT)
+
+            wait_for_done(
+                monitor.all_done,
+                self.file_upload_waiting_timeout,
+                progress_callback=monitor.log_remaining_uploads,
+                sleep_time=15,
+            )
+
         self.file_upload_manager.join()
         LOGGER.debug("Upload threads %r", self.file_upload_manager)
 
@@ -967,3 +1046,12 @@ def main_upload(archives, force_reupload):
     LOGGER.info("Number of uploaded experiments: %s", upload_count)
     if fail_count > 0:
         LOGGER.info("Number of failed experiment uploads: %s", fail_count)
+
+
+def _create_zip_file(directory, name):
+    file_path = _offline_zip_path(directory, name)
+    return ZipFile(file_path, "w", allowZip64=True)
+
+
+def _offline_zip_path(directory, name):
+    return os.path.join(directory, "%s.zip" % name)
