@@ -13,8 +13,11 @@
 # *******************************************************
 import logging
 import multiprocessing.pool
+import os
+import threading
+from functools import wraps
 
-from ._typing import Tuple
+from ._typing import Callable, Tuple
 
 LOGGER = logging.getLogger(__name__)
 
@@ -33,3 +36,25 @@ def get_thread_pool(worker_cpu_ratio):
     pool_size = min(64, cpu_count * worker_cpu_ratio)
 
     return (pool_size, cpu_count, multiprocessing.pool.ThreadPool(processes=pool_size))
+
+
+def synchronised(func):
+    # type: (Callable) -> Callable
+    """The decorator to make particular function synchronized"""
+    func.__lock__ = threading.Lock()  # type: ignore
+
+    @wraps(func)
+    def synced_func(*args, **kws):
+        with func.__lock__:  # type: ignore
+            return func(*args, **kws)
+
+    return synced_func
+
+
+@synchronised
+def makedirs_synchronized(name, exist_ok=False):
+    """
+    Replacement for Python2's version lacking exist_ok
+    """
+    if not os.path.exists(name) or not exist_ok:
+        os.makedirs(name)
