@@ -145,8 +145,7 @@ class Predictor(object):
             self.api = api if api else get_optimizer_api_client(self.experiment.api_key)
 
     def _set_mode(self, optimizer_id=None):
-        """ Try to use the global mode but fallback on the local mode if not optimizer_id could be found. Show a WARNING in that case.
-        """
+        """Try to use the global mode but fallback on the local mode if not optimizer_id could be found. Show a WARNING in that case."""
         try:
             self._setup_global_mode(optimizer_id)
 
