@@ -90,7 +90,8 @@ def get_parser_arguments(parser):
 def main(args):
     # Called via `comet init ...`
     parser = argparse.ArgumentParser(
-        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter,
+        description=__doc__,
+        formatter_class=argparse.RawDescriptionHelpFormatter,
     )
     get_parser_arguments(parser)
     parsed_args, remaining = parser.parse_known_args(args)
@@ -107,9 +108,9 @@ def init(parsed_args, remaining):
 
 def init_cookiecutter(parsed_args):
     try:
-        from cookiecutter.main import cookiecutter
-        from cookiecutter.exceptions import OutputDirExistsException
         from click.exceptions import Abort
+        from cookiecutter.exceptions import OutputDirExistsException
+        from cookiecutter.main import cookiecutter
     except ImportError:
         LOGGER.error(
             'Please install cookiecutter with `pip install "cookiecutter>1.7.0"`'
