@@ -109,7 +109,10 @@ def get_parser_arguments(parser):
     )
     list_parser.set_defaults(sub_func=list_models)
     list_parser.add_argument(
-        "-w", "--workspace", help="the workspace name to list", required=True,
+        "-w",
+        "--workspace",
+        help="the workspace name to list",
+        required=True,
     )
 
 
