@@ -14,6 +14,7 @@
 
 from __future__ import print_function
 
+import atexit
 import calendar
 import getpass
 import io
@@ -28,6 +29,7 @@ import re
 import sys
 import tempfile
 import time
+import uuid
 from datetime import datetime
 
 import six
@@ -48,8 +50,9 @@ from ._typing import (
     Tuple,
     Union,
 )
-from .compat_utils import Mapping, StringIO
+from .compat_utils import Mapping, StringIO, json_dump
 from .convert_utils import convert_tensor_to_numpy
+from .json_encoder import NestedEncoder
 from .logging_messages import DATAFRAME_PROFILE_ERROR, MISSING_PANDAS_PROFILING
 
 LOGGER = logging.getLogger(__name__)
@@ -77,7 +80,7 @@ INFINITY = float("inf")
 
 
 def fix_special_floats(value, _inf=INFINITY, _neginf=-INFINITY):
-    """ Fix out of bounds floats (like infinity and -infinity) and Not A
+    """Fix out of bounds floats (like infinity and -infinity) and Not A
     Number.
     Returns either a fixed value that could be JSON encoded or the original
     value.
@@ -147,8 +150,7 @@ def is_iterable(value):
 
 
 def is_list_like(value):
-    """ Check if the value is a list-like
-    """
+    """Check if the value is a list-like"""
     if is_iterable(value) and not isinstance(value, six.string_types):
         return True
 
@@ -165,17 +167,15 @@ def to_utf8(str_or_bytes):
 
 def local_timestamp():
     # type: () -> int
-    """ Return a timestamp in a format expected by the backend (milliseconds)
-    """
+    """Return a timestamp in a format expected by the backend (milliseconds)"""
     now = datetime.utcnow()
     timestamp_in_seconds = calendar.timegm(now.timetuple()) + (now.microsecond / 1e6)
     timestamp_in_milliseconds = int(timestamp_in_seconds * 1000)
     return timestamp_in_milliseconds
 
 
-def wait_for_empty(check_function, timeout, progress_callback=None, sleep_time=1):
-    """ Wait up to TIMEOUT seconds for the messages queue to be empty
-    """
+def wait_for_done(check_function, timeout, progress_callback=None, sleep_time=1):
+    """Wait up to TIMEOUT seconds for the check function to return True"""
     end_time = time.time() + timeout
     while check_function() is False and time.time() < end_time:
         if progress_callback is not None:
@@ -185,7 +185,7 @@ def wait_for_empty(check_function, timeout, progress_callback=None, sleep_time=1
         # before sleep_time has elapsed:
         end_sleep_time = time.time() + sleep_time
         while check_function() is False and time.time() < end_sleep_time:
-            time.sleep(sleep_time / 20)
+            time.sleep(sleep_time / 20.0)
 
 
 def read_unix_packages(package_status_file="/var/lib/dpkg/status"):
@@ -573,7 +573,7 @@ def rescale_array(array, old_range, new_range, dtype):
     old_delta = float(old_max - old_min)
     new_delta = float(new_max - new_min)
     if old_delta == 0:
-        return ((array - old_min) + (new_min + new_max) / 2).astype(dtype)
+        return ((array - old_min) + (new_min + new_max) / 2.0).astype(dtype)
     else:
         return (new_min + (array - old_min) * new_delta / old_delta).astype(dtype)
 
@@ -671,22 +671,24 @@ def data_to_fp(data):
     return fp
 
 
+ONE_KBYTE = float(1024)
+ONE_MBYTE = float(1024 * 1024)
+ONE_GBYTE = float(1024 * 1024 * 1024)
+
+
 def format_bytes(size):
-    # type: (int) -> str
+    # type: (float) -> str
     """
     Given a size in bytes, return a sort string representation.
     """
-    kbytes = size // 1024
-    mbytes = kbytes // 1024
-    gbytes = mbytes // 1024
-    if gbytes:
-        return "%s %s" % (gbytes, "GB")
-    elif mbytes:
-        return "%s %s" % (mbytes, "MB")
-    elif kbytes:
-        return "%s %s" % (kbytes, "KB")
+    if size >= ONE_GBYTE:
+        return "%.2f %s" % (size / ONE_GBYTE, "GB")
+    elif size >= ONE_MBYTE:
+        return "%.2f %s" % (size / ONE_MBYTE, "MB")
+    elif size >= ONE_KBYTE:
+        return "%.2f %s" % (size / ONE_KBYTE, "KB")
     else:
-        return "%s %s" % (size, "bytes")
+        return "%d %s" % (size, "bytes")
 
 
 class Embedding(object):
@@ -1057,8 +1059,7 @@ class Histogram(object):
 
     def is_empty(self):
         # type: () -> bool
-        """ Check if the Histogram is empty. Return True if empty, False otherwise
-        """
+        """Check if the Histogram is empty. Return True if empty, False otherwise"""
         # If the Histogram contains at least one value, at least one element of
         # self.counts will be not null
         return not any(self.counts)
@@ -1272,7 +1273,7 @@ class Histogram(object):
 
 def write_numpy_array_as_wav(numpy_array, sample_rate, file_object):
     # type: (Any, int, IO) -> None
-    """ Convert a numpy array to a WAV file using the given sample_rate and
+    """Convert a numpy array to a WAV file using the given sample_rate and
     write it to the file object
     """
     try:
@@ -1549,7 +1550,8 @@ def fast_flatten(items):
                 return numpy.array(flatten(items))
     else:
         log_once_at_level(
-            logging.INFO, "numpy not installed; using a slower flatten",
+            logging.INFO,
+            "numpy not installed; using a slower flatten",
         )
         return flatten(items)
 
@@ -1638,11 +1640,13 @@ def clean_and_check_root_relative_path(root, relative_path):
     if not root.endswith(os.sep):
         root = root + os.sep
 
-    joined_path = os.path.join(root, relative_path)
+    real_root = os.path.realpath(root)
+
+    joined_path = os.path.join(real_root, relative_path)
     resolved_path = os.path.realpath(joined_path)
 
-    if not resolved_path.startswith(root):
-        raise ValueError("Final path %r is outside of %r" % (resolved_path, root))
+    if not resolved_path.startswith(real_root):
+        raise ValueError("Final path %r is outside of %r" % (resolved_path, real_root))
 
     return resolved_path
 
@@ -1909,3 +1913,32 @@ class IterationProgressCallback(object):
                     self._callback()
                 except Exception:
                     LOGGER.debug("Error calling the progress callback", exc_info=True)
+
+
+def atexit_unregister(func):
+    # type: (Any) -> None
+    if six.PY2:
+        to_delete = []
+
+        # First iterate to detect exithandlers to remove
+        for exithandlers in atexit._exithandlers:
+            if exithandlers[0] == func:
+                to_delete.append(exithandlers)
+
+        # Then remove them but we cannot assume that the list hasn't changed, so uses remove which is inefficient
+        for exithandler_to_delete in to_delete:
+            try:
+                atexit._exithandlers.remove(exithandler_to_delete)
+            except ValueError:
+                pass
+    else:
+        return atexit.unregister(func)
+
+
+def generate_guid():
+    """Generate a GUID"""
+    return uuid.uuid4().hex
+
+
+def compact_json_dump(data, fp):
+    return json_dump(data, fp, sort_keys=True, separators=(",", ":"), cls=NestedEncoder)
